
        Here's the GUS autodetection code I promised to send a while
        ago. This only checks the ULTRASND environment variable, which
        is the only reliable way to do the autodetecting. The auto-
        detection routines in many demos use direct outputting to
        unknown hardware ports. If there happens to be a net card
        or a S3 graphics card it might screw up the computer. This
        is why everyone should start using the environment variable
        for the autodetection.
 
        The following code is combined from some extracts from our
        player routine. It's not perfect, it won't work if you have
        any extra spaces in the environment variable setting. And
        it also detects only addresses in form 2x0h. This is not
        a problem now, but someday there might be GUSses that could
        have some other addresses as well.
 
 
;---------------------------------------------------------------------------
 
        BPtr    EQU Byte Ptr
	WPtr	EQU Word Ptr
        DPtr    EQU Dword Ptr
 
DeviceInfo      Label   Word            ; Device info block.
BaseAddr	DW	220h
IRQ		DW	5
DMA             DW      5
 
EnvName         DB      'ULTRASND='     ; Ultrasound env. variable name.
EnvNameLen      EQU     $-EnvName       ; Length of the name.
 
;---------------------------------------------------------------------------
ParseEnvironment        Proc
 
        push    ds                   ; Save DS and ES during this procedure.
        push    es
        mov     ax, cs               ; Let DS point to current code segment.
        mov     ds, ax
 
        mov     ah, 62h                 ; DOS function 62h - Get PSP segment.
        int     21h                     ; Invoke DOS function dispatcher.
        mov     es, bx                  ; Load the PSP segment into ES.
        mov     ax, es:[2Ch]            ; Fetch the environment segment.
        mov     es, ax               ; Load the environment segment into ES.
 
        xor     di, di                  ; Zero out DI.
 
LongLoop:
        xor     cx, cx                  ; Zero out CX.
        mov     si, offset EnvName      ; Offset of needed env. variable name.
        lodsb                           ; Get first character of env. name.
MidLoop:
        cmp     Bptr es:[di], 0         ; A separator null btwn variables?
        jne     NotZero                 ; It not, it's beginning of a var.
        cmp     Bptr es:[di-1], 0       ; Already 2 nulls found?
        jz      EnvNotFound             ; If yes, then not found.
  NotZero:
        scasb                           ; Does the env.var. name match?
        jne     LongLoop                ; No, loop to compare next address.
        inc     cx                      ; Increment matching character count.
        cmp     cx, EnvNameLen          ; Already found enough?
        jae     LoopOver                ; If yes, go parse the variable.
        lodsb                           ; No, continue search.
        jmp     MidLoop
LoopOver:
        movzx   ax, Bptr es:[di+1]      ; Get the ASCII number 2x0 into AL.
        sub     al, 30h                 ; Make it a binary number.
        shl     ax, 4                   ; Move it to the second nibble.
        add     ax, 200h                ; Add 200h to the address value.
        mov     BaseAddr, ax            ; Store the base address.
        movzx   ax, Bptr es:[di+4]      ; Get the DMA channel number.
        sub     al, 30h                 ; Convert ASCII number to binary.
        mov     DMA, ax                 ; Store the DMA channel.
        cmp     Bptr es:[di+8], '1'     ; Is '1' the first char of IRQ?
        je      OneTeen                 ; If yes, it's either 1 or >=10.
        mov     al, es:[di+8]           ; No, it's 0 or 2-9.
        sub     al, 30h                 ; Convert ASCII number to binary.
        jmp     FoundIRQ                ; Thus, IRQ found.
OneTeen:
        cmp     Bptr es:[di+9], ','     ; Does a comma follow the '1'?
        jne     Teen                    ; If not, it's >=10.
        mov     al, 1                   ; Yes, it must be 1.
        jmp     FoundIRQ                ; Thus, IRQ found to be 1.
Teen:   mov     al, es:[di+9]           ; Fetch the second number.
        sub     al, 30h                 ; Convert ASCII number to binary.
        add     al, 10                  ; It's 10 or more, add 10.
FoundIRQ:
        xor     ah, ah
        mov     IRQ, ax                 ; IRQ vector succesfully found.
 
EnvNotFound:                            ; Not found? Use defaults.
        pop     es                      ; Restore saved ES and DS.
        pop     ds
        ret
 
ParseEnvironment        EndP
;----------------------------------------------------------------------------

  ______________________________
 /       /       /       \      \        Jani Mattsson, Turku Finland
/_______/_______/_________\______\       Handle: Saint of EMF
\______/   ____/          /   ___/       E-mail: jpm@mits.mdata.fi
  \___/   ____/   /  /   /   __/
    \/_______/___/__/___/____/ -hJ-      EMF E-mail: emf@mea.cc.utu.fi
